import jarvis
from discord.ext import commands
from discord_slash import cog_ext
from jarvis import jarvis_self, config, logo
from jarvis.utils import convert_bytesize, build_embed, get_repo_hash
from jarvis.utils.field import Field


class UtilCog(commands.Cog):
    """
    Utility functions for J.A.R.V.I.S.

    Mostly system utility functions, but may change over time
    """

    def __init__(self, bot):
        self.bot = bot
        self.config = config.get_config()

    async def _status(self, ctx):
        title = "J.A.R.V.I.S. Status"
        desc = "All systems online"
        color = "#98CCDA"
        fields = []
        with jarvis_self.oneshot():
            fields.append(Field("CPU Usage", jarvis_self.cpu_percent()))
            fields.append(
                Field(
                    "RAM Usage",
                    convert_bytesize(jarvis_self.memory_info().rss),
                )
            )
            fields.append(Field("PID", jarvis_self.pid))
            fields.append(Field("Version", jarvis.__version__, False))
            fields.append(Field("Git Hash", get_repo_hash(), False))
            embed = build_embed(
                title=title, description=desc, fields=fields, color=color
            )
            await ctx.send(embed=embed)

    @cog_ext.cog_slash(
        name="status",
        description="Retrieve J.A.R.V.I.S. status",
        guild_ids=[578757004059738142],
    )
    async def _status_slash(self, ctx):
        await self._status(ctx)

    @commands.command(
        name="status", description="Retrieve J.A.R.V.I.S. status"
    )
    async def _status_pre(self, ctx):
        await self._status(ctx)

    async def _logo(self, ctx):
        lo = logo.get_logo(self.config.logo)
        await ctx.send(f"```\n{lo}\n```")

    @commands.command(name="logo", help="Get the current logo")
    async def _logo_pre(self, ctx):
        await self._logo(ctx)

    @cog_ext.cog_slash(
        name="logo",
        description="Get the current logo",
        guild_ids=[578757004059738142],
    )
    async def _logo_slash(self, ctx):
        await self._logo(ctx)


def setup(bot):
    bot.add_cog(UtilCog(bot))
